@@ -1,7 +1,6 @@
 /* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil -*- */
+/* kdc/do_tgs_req.c - KDC Routines to deal with TGS_REQ's */
 /*
- * kdc/do_tgs_req.c
- *
  * Copyright 1990,1991,2001,2007,2008,2009 by the Massachusetts Institute of Technology.
  * All Rights Reserved.
  *
@@ -23,9 +22,6 @@
  * M.I.T. makes no representations about the suitability of
  * this software for any purpose.  It is provided "as is" without express
  * or implied warranty.
- *
- *
- * KDC Routines to deal with TGS_REQ's
  */
 /*
  * Copyright (c) 2006-2008, Novell, Inc.
@@ -77,7 +73,7 @@ find_alternate_tgs(krb5_kdc_req *,krb5_db_entry **);
 
 static krb5_error_code
 prepare_error_tgs(struct kdc_request_state *, krb5_kdc_req *,krb5_ticket *,int,
-                  krb5_principal,krb5_data **,const char *, krb5_data *);
+                  krb5_principal,krb5_data **,const char *, krb5_pa_data **);
 
 static krb5_int32
 prep_reprocess_req(krb5_kdc_req *,krb5_principal *);
@@ -127,12 +123,11 @@ process_tgs_req(krb5_data *pkt, const krb5_fulladdr *from,
     struct kdc_request_state *state = NULL;
     krb5_pa_data *pa_tgs_req; /*points into request*/
     krb5_data scratch;
-    krb5_data e_data = empty_data(); /* backend-provided error data */
+    krb5_pa_data **e_data = NULL;
 
     reply.padata = 0; /* For cleanup handler */
     reply_encpart.enc_padata = 0;
     enc_tkt_reply.authorization_data = NULL;
-    e_data.data = NULL;
 
     session_key.contents = NULL;
 
@@ -181,7 +176,7 @@ process_tgs_req(krb5_data *pkt, const krb5_fulladdr *from,
     scratch.length = pa_tgs_req->length;
     scratch.data = (char *) pa_tgs_req->contents;
     errcode = kdc_find_fast(&request, &scratch, subkey,
-                            header_ticket->enc_part2->session, state);
+                            header_ticket->enc_part2->session, state, NULL);
     if (errcode !=0) {
         status = "kdc_find_fast";
         goto cleanup;
@@ -689,7 +684,8 @@ tgt_again:
                               subject_tkt,
                               &enc_tkt_reply);
     if (errcode) {
-        krb5_klog_syslog(LOG_INFO, "TGS_REQ : handle_authdata (%d)", errcode);
+        krb5_klog_syslog(LOG_INFO, _("TGS_REQ : handle_authdata (%d)"),
+                         errcode);
         status = "HANDLE_AUTHDATA";
         goto cleanup;
     }
@@ -756,29 +752,24 @@ tgt_again:
         if (errcode == 0) {
             setflag (enc_tkt_reply.flags, TKT_FLG_TRANSIT_POLICY_CHECKED);
         } else if (errcode == KRB5KRB_AP_ERR_ILL_CR_TKT)
-            krb5_klog_syslog (LOG_INFO,
-                              "bad realm transit path from '%s' to '%s' "
-                              "via '%.*s%s'",
-                              cname ? cname : "<unknown client>",
-                              sname ? sname : "<unknown server>",
-                              tlen,
-                              enc_tkt_reply.transited.tr_contents.data,
-                              tdots);
+            krb5_klog_syslog(LOG_INFO, _("bad realm transit path from '%s' "
+                                         "to '%s' via '%.*s%s'"),
+                             cname ? cname : "<unknown client>",
+                             sname ? sname : "<unknown server>", tlen,
+                             enc_tkt_reply.transited.tr_contents.data, tdots);
         else {
             emsg = krb5_get_error_message(kdc_context, errcode);
-            krb5_klog_syslog (LOG_ERR,
-                              "unexpected error checking transit from "
-                              "'%s' to '%s' via '%.*s%s': %s",
-                              cname ? cname : "<unknown client>",
-                              sname ? sname : "<unknown server>",
-                              tlen,
-                              enc_tkt_reply.transited.tr_contents.data,
-                              tdots, emsg);
+            krb5_klog_syslog(LOG_ERR, _("unexpected error checking transit "
+                                        "from '%s' to '%s' via '%.*s%s': %s"),
+                             cname ? cname : "<unknown client>",
+                             sname ? sname : "<unknown server>", tlen,
+                             enc_tkt_reply.transited.tr_contents.data, tdots,
+                             emsg);
             krb5_free_error_message(kdc_context, emsg);
             emsg = NULL;
         }
     } else
-        krb5_klog_syslog (LOG_INFO, "not checking transit path");
+        krb5_klog_syslog(LOG_INFO, _("not checking transit path"));
     if (reject_bad_transit
         && !isflagset (enc_tkt_reply.flags, TKT_FLG_TRANSIT_POLICY_CHECKED)) {
         errcode = KRB5KDC_ERR_POLICY;
@@ -938,7 +929,7 @@ cleanup:
 
         retval = prepare_error_tgs(state, request, header_ticket, errcode,
                                    (server != NULL) ? server->princ : NULL,
-                                   response, status, &e_data);
+                                   response, status, e_data);
         if (got_err) {
             krb5_free_error_message (kdc_context, status);
             status = 0;
@@ -978,7 +969,7 @@ cleanup:
         krb5_free_pa_data(kdc_context, reply_encpart.enc_padata);
     if (enc_tkt_reply.authorization_data != NULL)
         krb5_free_authdata(kdc_context, enc_tkt_reply.authorization_data);
-    krb5_free_data_contents(kdc_context, &e_data);
+    krb5_free_pa_data(kdc_context, e_data);
 
     return retval;
 }
@@ -988,11 +979,11 @@ prepare_error_tgs (struct kdc_request_state *state,
                    krb5_kdc_req *request, krb5_ticket *ticket, int error,
                    krb5_principal canon_server,
                    krb5_data **response, const char *status,
-                   krb5_data *e_data)
+                   krb5_pa_data **e_data)
 {
     krb5_error errpkt;
     krb5_error_code retval = 0;
-    krb5_data *scratch, *fast_edata = NULL;
+    krb5_data *scratch, *e_data_asn1 = NULL, *fast_edata = NULL;
 
     errpkt.ctime = request->nonce;
     errpkt.cusec = 0;
@@ -1014,20 +1005,33 @@ prepare_error_tgs (struct kdc_request_state *state,
         free(errpkt.text.data);
         return ENOMEM;
     }
-    errpkt.e_data = *e_data;
+
+    if (e_data != NULL) {
+        retval = encode_krb5_padata_sequence(e_data, &e_data_asn1);
+        if (retval) {
+            free(scratch);
+            free(errpkt.text.data);
+            return retval;
+        }
+        errpkt.e_data = *e_data_asn1;
+    } else
+        errpkt.e_data = empty_data();
+
     if (state) {
-        retval = kdc_fast_handle_error(kdc_context, state, request, NULL,
+        retval = kdc_fast_handle_error(kdc_context, state, request, e_data,
                                        &errpkt, &fast_edata);
     }
     if (retval) {
         free(scratch);
         free(errpkt.text.data);
+        krb5_free_data(kdc_context, e_data_asn1);
         return retval;
     }
     if (fast_edata)
         errpkt.e_data = *fast_edata;
     retval = krb5_mk_error(kdc_context, &errpkt, scratch);
     free(errpkt.text.data);
+    krb5_free_data(kdc_context, e_data_asn1);
     krb5_free_data(kdc_context, fast_edata);
     if (retval)
         free(scratch);
@@ -1137,8 +1141,7 @@ prep_reprocess_req(krb5_kdc_req *request, krb5_principal *krbtgt_princ)
             retval = ENOMEM;
             goto cleanup;
         }
-        if (comp1->data != NULL)
-            memcpy(comp1_str, comp1->data, comp1->length);
+        strlcpy(comp1_str,comp1->data,comp1->length+1);
 
         if ((krb5_princ_type(kdc_context, request->server) == KRB5_NT_SRV_HST ||
              krb5_princ_type(kdc_context, request->server) == KRB5_NT_SRV_INST ||
@@ -1161,8 +1164,7 @@ prep_reprocess_req(krb5_kdc_req *request, krb5_principal *krbtgt_princ)
                 retval = ENOMEM;
                 goto cleanup;
             }
-            if (comp2->data != NULL)
-                memcpy(temp_buf, comp2->data, comp2->length);
+            strlcpy(temp_buf, comp2->data,comp2->length+1);
             retval = krb5int_get_domain_realm_mapping(kdc_context, temp_buf, &realms);
             free(temp_buf);
             if (retval) {
@@ -1174,11 +1176,7 @@ prep_reprocess_req(krb5_kdc_req *request, krb5_principal *krbtgt_princ)
                 retval = KRB5KRB_AP_ERR_BADMATCH;
                 goto cleanup;
             }
-            /* Don't return a referral to the null realm or the service
-             * realm. */
-            if (realms[0] == 0 ||
-                data_eq_string(request->server->realm, realms[0])) {
-                free(realms[0]);
+            if (realms[0] == 0) {
                 free(realms);
                 retval = KRB5KRB_AP_ERR_BADMATCH;
                 goto cleanup;
