@@ -1,7 +1,6 @@
 /* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil -*- */
+/* kdc/kdc_util.c - Utility functions for the KDC implementation */
 /*
- * kdc/kdc_util.c
- *
  * Copyright 1990,1991,2007,2008,2009 by the Massachusetts Institute of Technology.
  * All Rights Reserved.
  *
@@ -23,9 +22,6 @@
  * M.I.T. makes no representations about the suitability of
  * this software for any purpose.  It is provided "as is" without express
  * or implied warranty.
- *
- *
- * Utility functions for the KDC implementation.
  */
 /*
  * Copyright (c) 2006-2008, Novell, Inc.
@@ -155,7 +151,7 @@ is_cross_tgs_principal(krb5_const_principal principal)
             data_eq_string(*krb5_princ_component(kdc_context, principal, 0),
                            KRB5_TGS_NAME) &&
             !data_eq(*krb5_princ_component(kdc_context, principal, 1),
-                     *krb5_princ_realm(kcd_context, principal)));
+                     *krb5_princ_realm(kdc_context, principal)));
 }
 
 /*
@@ -229,7 +225,7 @@ kdc_process_tgs_req(krb5_kdc_req *request, const krb5_fulladdr *from,
 
     if (isflagset(apreq->ap_options, AP_OPTS_USE_SESSION_KEY) ||
         isflagset(apreq->ap_options, AP_OPTS_MUTUAL_REQUIRED)) {
-        krb5_klog_syslog(LOG_INFO, "TGS_REQ: SESSION KEY or MUTUAL");
+        krb5_klog_syslog(LOG_INFO, _("TGS_REQ: SESSION KEY or MUTUAL"));
         retval = KRB5KDC_ERR_POLICY;
         goto cleanup;
     }
@@ -288,10 +284,10 @@ kdc_process_tgs_req(krb5_kdc_req *request, const krb5_fulladdr *from,
                                                  &authenticator)))
         goto cleanup_auth_context;
 
-    retval = krb5int_find_authdata(kdc_context,
-                                   (*ticket)->enc_part2->authorization_data,
-                                   authenticator->authorization_data,
-                                   KRB5_AUTHDATA_FX_ARMOR, &authdata);
+    retval = krb5_find_authdata(kdc_context,
+                                (*ticket)->enc_part2->authorization_data,
+                                authenticator->authorization_data,
+                                KRB5_AUTHDATA_FX_ARMOR, &authdata);
     if (retval != 0)
         goto cleanup_authenticator;
     if (authdata&& authdata[0]) {
@@ -315,7 +311,7 @@ kdc_process_tgs_req(krb5_kdc_req *request, const krb5_fulladdr *from,
         !find_pa_data(request->padata, KRB5_PADATA_FOR_USER)) {
         if (is_local_principal((*ticket)->enc_part2->client)) {
             /* someone in a foreign realm claiming to be local */
-            krb5_klog_syslog(LOG_INFO, "PROCESS_TGS: failed lineage check");
+            krb5_klog_syslog(LOG_INFO, _("PROCESS_TGS: failed lineage check"));
             retval = KRB5KDC_ERR_POLICY;
             goto cleanup_authenticator;
         }
@@ -385,8 +381,8 @@ kdc_get_server_key(krb5_ticket *ticket, unsigned int flags,
         char *sname;
         if (!krb5_unparse_name(kdc_context, ticket->server, &sname)) {
             limit_string(sname);
-            krb5_klog_syslog(LOG_ERR,"TGS_REQ: UNKNOWN SERVER: server='%s'",
-                             sname);
+            krb5_klog_syslog(LOG_ERR,
+                             _("TGS_REQ: UNKNOWN SERVER: server='%s'"), sname);
             free(sname);
         }
         return KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN;
@@ -869,7 +865,7 @@ check_anon(krb5_context context, krb5_principal client, krb5_principal server)
 int
 validate_as_request(register krb5_kdc_req *request, krb5_db_entry client,
                     krb5_db_entry server, krb5_timestamp kdc_time,
-                    const char **status, krb5_data *e_data)
+                    const char **status, krb5_pa_data ***e_data)
 {
     int errcode;
     krb5_error_code ret;
@@ -1172,7 +1168,7 @@ fetch_asn1_field(unsigned char *astream, unsigned int level,
 int
 validate_tgs_request(register krb5_kdc_req *request, krb5_db_entry server,
                      krb5_ticket *ticket, krb5_timestamp kdc_time,
-                     const char **status, krb5_data *e_data)
+                     const char **status, krb5_pa_data ***e_data)
 {
     int errcode;
     int st_idx = 0;
@@ -2087,9 +2083,8 @@ kdc_process_s4u2self_req(krb5_context context,
      */
     if (is_local_principal((*s4u_x509_user)->user_id.user)) {
         krb5_db_entry no_server;
-        krb5_data e_data;
+        krb5_pa_data **e_data = NULL;
 
-        e_data.data = NULL;
         code = krb5_db_get_principal(context, (*s4u_x509_user)->user_id.user,
                                      KRB5_KDB_FLAG_INCLUDE_PAC, &princ);
         if (code == KRB5_KDB_NOENTRY) {
@@ -2106,7 +2101,7 @@ kdc_process_s4u2self_req(krb5_context context,
                                    no_server, kdc_time, status, &e_data);
         if (code) {
             krb5_db_free_principal(context, princ);
-            krb5_free_data_contents(context, &e_data);
+            krb5_free_pa_data(context, e_data);
             return code;
         }
 
@@ -2255,13 +2250,13 @@ log_as_req(const krb5_fulladdr *from,
         /* success */
         char rep_etypestr[128];
         rep_etypes2str(rep_etypestr, sizeof(rep_etypestr), reply);
-        krb5_klog_syslog(LOG_INFO,
-                         "AS_REQ (%s) %s: ISSUE: authtime %d, %s, %s for %s",
+        krb5_klog_syslog(LOG_INFO, _("AS_REQ (%s) %s: ISSUE: authtime %d, %s, "
+                                     "%s for %s"),
                          ktypestr, fromstring, authtime,
                          rep_etypestr, cname2, sname2);
     } else {
         /* fail */
-        krb5_klog_syslog(LOG_INFO, "AS_REQ (%s) %s: %s: %s for %s%s%s",
+        krb5_klog_syslog(LOG_INFO, _("AS_REQ (%s) %s: %s: %s for %s%s%s"),
                          ktypestr, fromstring, status,
                          cname2, sname2, emsg ? ", " : "", emsg ? emsg : "");
     }
@@ -2309,31 +2304,28 @@ log_tgs_req(const krb5_fulladdr *from,
        name (useful), and doesn't log ktypestr (probably not
        important).  */
     if (errcode != KRB5KDC_ERR_SERVER_NOMATCH) {
-        krb5_klog_syslog(LOG_INFO,
-                         "TGS_REQ (%s) %s: %s: authtime %d, %s%s %s for %s%s%s",
-                         ktypestr,
-                         fromstring, status, authtime,
-                         rep_etypestr,
+        krb5_klog_syslog(LOG_INFO, _("TGS_REQ (%s) %s: %s: authtime %d, %s%s "
+                                     "%s for %s%s%s"),
+                         ktypestr, fromstring, status, authtime, rep_etypestr,
                          !errcode ? "," : "",
                          cname ? cname : "<unknown client>",
                          sname ? sname : "<unknown server>",
-                         errcode ? ", " : "",
-                         errcode ? emsg : "");
+                         errcode ? ", " : "", errcode ? emsg : "");
         if (s4u_name) {
             assert(isflagset(c_flags, KRB5_KDB_FLAG_PROTOCOL_TRANSITION) ||
                    isflagset(c_flags, KRB5_KDB_FLAG_CONSTRAINED_DELEGATION));
             if (isflagset(c_flags, KRB5_KDB_FLAG_PROTOCOL_TRANSITION))
                 krb5_klog_syslog(LOG_INFO,
-                                 "... PROTOCOL-TRANSITION s4u-client=%s",
+                                 _("... PROTOCOL-TRANSITION s4u-client=%s"),
                                  s4u_name);
             else if (isflagset(c_flags, KRB5_KDB_FLAG_CONSTRAINED_DELEGATION))
                 krb5_klog_syslog(LOG_INFO,
-                                 "... CONSTRAINED-DELEGATION s4u-client=%s",
+                                 _("... CONSTRAINED-DELEGATION s4u-client=%s"),
                                  s4u_name);
         }
     } else
-        krb5_klog_syslog(LOG_INFO,
-                         "TGS_REQ %s: %s: authtime %d, %s for %s, 2nd tkt client %s",
+        krb5_klog_syslog(LOG_INFO, _("TGS_REQ %s: %s: authtime %d, %s for %s, "
+                                     "2nd tkt client %s"),
                          fromstring, status, authtime,
                          cname ? cname : "<unknown client>",
                          sname ? sname : "<unknown server>",
@@ -2349,10 +2341,10 @@ log_tgs_alt_tgt(krb5_principal p)
     char *sname;
     if (krb5_unparse_name(kdc_context, p, &sname)) {
         krb5_klog_syslog(LOG_INFO,
-                         "TGS_REQ: issuing alternate <un-unparseable> TGT");
+                         _("TGS_REQ: issuing alternate <un-unparseable> TGT"));
     } else {
         limit_string(sname);
-        krb5_klog_syslog(LOG_INFO, "TGS_REQ: issuing TGT %s", sname);
+        krb5_klog_syslog(LOG_INFO, _("TGS_REQ: issuing TGT %s"), sname);
         free(sname);
     }
     /* OpenSolaris: audit_krb5kdc_tgs_req_alt_tgt(...) */
@@ -2464,7 +2456,7 @@ kdc_handle_protected_negotiation(krb5_data *req_pkt, krb5_kdc_req *request,
     krb5_data *out = NULL;
     krb5_pa_data pa, *pa_in;
     pa_in = krb5int_find_pa_data(kdc_context, request->padata,
-                              KRB5_ENCPADATA_REQ_ENC_PA_REP);
+                                 KRB5_ENCPADATA_REQ_ENC_PA_REP);
     if (pa_in == NULL)
         return 0;
     pa.magic = KV5M_PA_DATA;
