@@ -84,6 +84,10 @@
 #include "gssapiP_krb5.h"
 #include "mglueP.h"
 
+#ifndef NO_PASSWORD
+#include <pwd.h>
+#endif
+
 /** exported constants defined in gssapi_krb5{,_nx}.h **/
 
 /* these are bogus, but will compile */
@@ -341,7 +345,7 @@ static struct {
     }
 };
 
-static OM_uint32
+static OM_uint32 KRB5_CALLCONV
 krb5_gss_inquire_sec_context_by_oid (OM_uint32 *minor_status,
                                      const gss_ctx_id_t context_handle,
                                      const gss_OID desired_object,
@@ -363,9 +367,6 @@ krb5_gss_inquire_sec_context_by_oid (OM_uint32 *minor_status,
 
     *data_set = GSS_C_NO_BUFFER_SET;
 
-    if (!kg_validate_ctx_id(context_handle))
-        return GSS_S_NO_CONTEXT;
-
     ctx = (krb5_gss_ctx_id_rec *) context_handle;
 
     if (!ctx->established)
@@ -397,7 +398,7 @@ static struct {
 };
 #endif
 
-static OM_uint32
+static OM_uint32 KRB5_CALLCONV
 krb5_gss_inquire_cred_by_oid(OM_uint32 *minor_status,
                              const gss_cred_id_t cred_handle,
                              const gss_OID desired_object,
@@ -451,6 +452,7 @@ krb5_gss_inquire_cred_by_oid(OM_uint32 *minor_status,
 
 /*
  * gss_set_sec_context_option() methods
+ * (Disabled until we have something to populate the array.)
  */
 #if 0
 static struct {
@@ -460,13 +462,15 @@ static struct {
 };
 #endif
 
-static OM_uint32
+static OM_uint32 KRB5_CALLCONV
 krb5_gss_set_sec_context_option (OM_uint32 *minor_status,
                                  gss_ctx_id_t *context_handle,
                                  const gss_OID desired_object,
                                  const gss_buffer_t value)
 {
+#if 0
     size_t i;
+#endif
 
     if (minor_status == NULL)
         return GSS_S_CALL_INACCESSIBLE_WRITE;
@@ -479,15 +483,6 @@ krb5_gss_set_sec_context_option (OM_uint32 *minor_status,
     if (desired_object == GSS_C_NO_OID)
         return GSS_S_CALL_INACCESSIBLE_READ;
 
-    if (*context_handle != GSS_C_NO_CONTEXT) {
-        krb5_gss_ctx_id_rec *ctx;
-
-        if (!kg_validate_ctx_id(*context_handle))
-            return GSS_S_NO_CONTEXT;
-
-        ctx = (krb5_gss_ctx_id_rec *) context_handle;
-    }
-
 #if 0
     for (i = 0; i < sizeof(krb5_gss_set_sec_context_option_ops)/
              sizeof(krb5_gss_set_sec_context_option_ops[0]); i++) {
@@ -530,7 +525,7 @@ static struct {
     },
 };
 
-static OM_uint32
+static OM_uint32 KRB5_CALLCONV
 krb5_gssspi_set_cred_option(OM_uint32 *minor_status,
                             gss_cred_id_t *cred_handle,
                             const gss_OID desired_object,
@@ -598,7 +593,7 @@ static struct {
 #endif
 };
 
-static OM_uint32
+static OM_uint32 KRB5_CALLCONV
 krb5_gssspi_mech_invoke (OM_uint32 *minor_status,
                          const gss_OID desired_mech,
                          const gss_OID desired_object,
@@ -638,7 +633,7 @@ krb5_gssspi_mech_invoke (OM_uint32 *minor_status,
 #define GS2_IAKERB_SASL_NAME      "GS2-IAKERB"
 #define GS2_IAKERB_SASL_NAME_LEN  (sizeof(GS2_IAKERB_SASL_NAME) - 1)
 
-static OM_uint32
+static OM_uint32 KRB5_CALLCONV
 krb5_gss_inquire_mech_for_saslname(OM_uint32 *minor_status,
                                    const gss_buffer_t sasl_mech_name,
                                    gss_OID *mech_type)
@@ -652,8 +647,8 @@ krb5_gss_inquire_mech_for_saslname(OM_uint32 *minor_status,
             *mech_type = (gss_OID)gss_mech_krb5;
         return GSS_S_COMPLETE;
     } else if (sasl_mech_name->length == GS2_IAKERB_SASL_NAME_LEN &&
-        memcmp(sasl_mech_name->value,
-               GS2_IAKERB_SASL_NAME, GS2_IAKERB_SASL_NAME_LEN) == 0) {
+               memcmp(sasl_mech_name->value,
+                      GS2_IAKERB_SASL_NAME, GS2_IAKERB_SASL_NAME_LEN) == 0) {
         if (mech_type != NULL)
             *mech_type = (gss_OID)gss_mech_iakerb;
         return GSS_S_COMPLETE;
@@ -662,7 +657,7 @@ krb5_gss_inquire_mech_for_saslname(OM_uint32 *minor_status,
     return GSS_S_BAD_MECH;
 }
 
-static OM_uint32
+static OM_uint32 KRB5_CALLCONV
 krb5_gss_inquire_saslname_for_mech(OM_uint32 *minor_status,
                                    const gss_OID desired_mech,
                                    gss_buffer_t sasl_mech_name,
@@ -673,7 +668,8 @@ krb5_gss_inquire_saslname_for_mech(OM_uint32 *minor_status,
         if (!g_make_string_buffer(GS2_IAKERB_SASL_NAME, sasl_mech_name) ||
             !g_make_string_buffer("iakerb", mech_name) ||
             !g_make_string_buffer("Initial and Pass Through Authentication "
-                             "Kerberos Mechanism (IAKERB)", mech_description))
+                                  "Kerberos Mechanism (IAKERB)",
+                                  mech_description))
             goto fail;
     } else {
         if (!g_make_string_buffer(GS2_KRB5_SASL_NAME, sasl_mech_name) ||
@@ -691,7 +687,7 @@ fail:
     return GSS_S_FAILURE;
 }
 
-static OM_uint32
+static OM_uint32 KRB5_CALLCONV
 krb5_gss_inquire_attrs_for_mech(OM_uint32 *minor_status,
                                 gss_const_OID mech,
                                 gss_OID_set *mech_attrs,
@@ -708,10 +704,11 @@ krb5_gss_inquire_attrs_for_mech(OM_uint32 *minor_status,
     if (GSS_ERROR(major))
         goto cleanup;
 
-#define MA_SUPPORTED(ma)    do { \
-    major = gss_add_oid_set_member(minor_status, (gss_OID)ma, mech_attrs);  \
-    if (GSS_ERROR(major))                                                   \
-        goto cleanup;                                                       \
+#define MA_SUPPORTED(ma)    do {                                        \
+        major = gss_add_oid_set_member(minor_status, (gss_OID)ma,       \
+                                       mech_attrs);                     \
+        if (GSS_ERROR(major))                                           \
+            goto cleanup;                                               \
     } while (0)
 
     MA_SUPPORTED(GSS_C_MA_MECH_CONCRETE);
@@ -742,6 +739,86 @@ cleanup:
     return major;
 }
 
+static OM_uint32 KRB5_CALLCONV
+krb5_gss_localname(OM_uint32 *minor,
+                   const gss_name_t pname,
+                   const gss_const_OID mech_type,
+                   gss_buffer_t localname)
+{
+    krb5_context context;
+    krb5_error_code code;
+    krb5_gss_name_t kname;
+    char lname[BUFSIZ];
+
+    code = krb5_gss_init_context(&context);
+    if (code != 0) {
+        *minor = code;
+        return GSS_S_FAILURE;
+    }
+
+    kname = (krb5_gss_name_t)pname;
+
+    code = krb5_aname_to_localname(context, kname->princ,
+                                   sizeof(lname), lname);
+    if (code != 0) {
+        *minor = KRB5_NO_LOCALNAME;
+        krb5_free_context(context);
+        return GSS_S_FAILURE;
+    }
+
+
+    krb5_free_context(context);
+    localname->value = gssalloc_strdup(lname);
+    localname->length = strlen(lname);
+
+    return (code == 0) ? GSS_S_COMPLETE : GSS_S_FAILURE;
+}
+
+
+static OM_uint32 KRB5_CALLCONV
+krb5_gss_authorize_localname(OM_uint32 *minor,
+                             const gss_name_t pname,
+                             gss_const_buffer_t local_user,
+                             gss_const_OID name_type)
+{
+    krb5_context context;
+    krb5_error_code code;
+    krb5_gss_name_t kname;
+    char *user;
+    int user_ok;
+
+    if (name_type != GSS_C_NO_OID &&
+        !g_OID_equal(name_type, GSS_C_NT_USER_NAME)) {
+        return GSS_S_BAD_NAMETYPE;
+    }
+
+    kname = (krb5_gss_name_t)pname;
+
+    code = krb5_gss_init_context(&context);
+    if (code != 0) {
+        *minor = code;
+        return GSS_S_FAILURE;
+    }
+
+    user = k5alloc(local_user->length + 1, &code);
+    if (user == NULL) {
+        *minor = code;
+        krb5_free_context(context);
+        return GSS_S_FAILURE;
+    }
+
+    memcpy(user, local_user->value, local_user->length);
+    user[local_user->length] = '\0';
+
+    user_ok = krb5_kuserok(context, kname->princ, user);
+
+    free(user);
+    krb5_free_context(context);
+
+    *minor = 0;
+    return user_ok ? GSS_S_COMPLETE : GSS_S_UNAUTHORIZED;
+}
+
 static struct gss_config krb5_mechanism = {
     { GSS_MECH_KRB5_OID_LENGTH, GSS_MECH_KRB5_OID },
     NULL,
@@ -788,7 +865,11 @@ static struct gss_config krb5_mechanism = {
     krb5_gss_inquire_context,
     krb5_gss_internal_release_oid,
     krb5_gss_wrap_size_limit,
+    krb5_gss_localname,
+
+    krb5_gss_authorize_localname,
     krb5_gss_export_name,
+    krb5_gss_duplicate_name,
     krb5_gss_store_cred,
     krb5_gss_inquire_sec_context_by_oid,
     krb5_gss_inquire_cred_by_oid,
