@@ -1,7 +1,6 @@
 /* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil -*- */
+/* lib/krb5/ccache/cc_mslsa.c */
 /*
- * lib/krb5/ccache/cc_mslsa.c
- *
  * Copyright 2007 Secure Endpoints Inc.
  *
  * Copyright 2003,2004 by the Massachusetts Institute of Technology.
@@ -54,6 +53,8 @@
 #define UNICODE
 #define _UNICODE
 
+#include <ntstatus.h>
+#define WIN32_NO_STATUS
 #include "k5-int.h"
 #include "com_err.h"
 #include "cc-int.h"
@@ -71,7 +72,6 @@
 #endif
 #define _WIN32_WINNT 0x0600
 #include <ntsecapi.h>
-#include <ntstatus.h>
 
 
 /* The following two features can only be built using the version of the
@@ -337,23 +337,15 @@ UnicodeToANSI(LPTSTR lpInputString, LPSTR lpszOutputString, int nOutStringLen)
 
 static VOID
 WINAPI
-ANSIToUnicode(LPSTR lpInputString, LPTSTR lpszOutputString, int nOutStringLen)
+ANSIToUnicode(LPCSTR lpInputString, LPWSTR lpszOutputString, int nOutStringLen)
 {
 
     CPINFO CodePageInfo;
 
-    lstrcpy(lpszOutputString, (LPTSTR) lpInputString);
-
     GetCPInfo(CP_ACP, &CodePageInfo);
 
-    if (CodePageInfo.MaxCharSize > 1 || ((LPBYTE) lpInputString)[1] != '\0')
-    {
-        // Looks like ANSI or MultiByte, better translate it
-        MultiByteToWideChar(CP_ACP, 0, (LPCSTR) lpInputString, -1,
-                            (LPWSTR) lpszOutputString, nOutStringLen);
-    }
-    else
-        lstrcpy(lpszOutputString, (LPTSTR) lpInputString);
+    MultiByteToWideChar(CP_ACP, 0, lpInputString, -1,
+                        lpszOutputString, nOutStringLen);
 }  // ANSIToUnicode
 
 
@@ -669,7 +661,7 @@ does_retrieve_ticket_cache_ticket (void)
 
         pTicketRequest = (PKERB_RETRIEVE_TKT_REQUEST) LocalAlloc(LMEM_ZEROINIT, RequestSize);
         if (!pTicketRequest) {
-            CloseHandle(LogonHandle);
+            LsaDeregisterLogonProcess(LogonHandle);
             return FALSE;
         }
 
@@ -694,7 +686,7 @@ does_retrieve_ticket_cache_ticket (void)
         );
 
         LocalFree(pTicketRequest);
-        CloseHandle(LogonHandle);
+        LsaDeregisterLogonProcess(LogonHandle);
 
         if (FAILED(Status) || FAILED(SubStatus)) {
             if ( SubStatus == STATUS_NOT_SUPPORTED )
@@ -734,7 +726,7 @@ does_query_ticket_cache_ex2 (void)
 
         pCacheRequest = (PKERB_QUERY_TKT_CACHE_REQUEST) LocalAlloc(LMEM_ZEROINIT, RequestSize);
         if (!pCacheRequest) {
-            CloseHandle(LogonHandle);
+            LsaDeregisterLogonProcess(LogonHandle);
             return FALSE;
         }
 
@@ -752,7 +744,7 @@ does_query_ticket_cache_ex2 (void)
         );
 
         LocalFree(pCacheRequest);
-        CloseHandle(LogonHandle);
+        LsaDeregisterLogonProcess(LogonHandle);
 
         if (!(FAILED(Status) || FAILED(SubStatus))) {
             LsaFreeReturnBuffer(pCacheResponse);
@@ -2049,7 +2041,7 @@ krb5_lcc_resolve (krb5_context context, krb5_ccache *id, const char *residual)
 
     lid = (krb5_ccache) malloc(sizeof(struct _krb5_ccache));
     if (lid == NULL) {
-        CloseHandle(LogonHandle);
+        LsaDeregisterLogonProcess(LogonHandle);
         return KRB5_CC_NOMEM;
     }
 
@@ -2058,7 +2050,7 @@ krb5_lcc_resolve (krb5_context context, krb5_ccache *id, const char *residual)
     lid->data = (krb5_pointer) malloc(sizeof(krb5_lcc_data));
     if (lid->data == NULL) {
         free(lid);
-        CloseHandle(LogonHandle);
+        LsaDeregisterLogonProcess(LogonHandle);
         return KRB5_CC_NOMEM;
     }
 
@@ -2072,7 +2064,7 @@ krb5_lcc_resolve (krb5_context context, krb5_ccache *id, const char *residual)
     if (data->cc_name == NULL) {
         free(lid->data);
         free(lid);
-        CloseHandle(LogonHandle);
+        LsaDeregisterLogonProcess(LogonHandle);
         return KRB5_CC_NOMEM;
     }
     strcpy(data->cc_name, residual);
@@ -2094,7 +2086,7 @@ krb5_lcc_resolve (krb5_context context, krb5_ccache *id, const char *residual)
         free(data->cc_name);
         free(lid->data);
         free(lid);
-        CloseHandle(LogonHandle);
+        LsaDeregisterLogonProcess(LogonHandle);
         return KRB5_FCC_NOFILE;
     }
 
@@ -2169,7 +2161,9 @@ krb5_lcc_close(krb5_context context, krb5_ccache id)
         data = (krb5_lcc_data *) id->data;
 
         if (data) {
-            CloseHandle(data->LogonHandle);
+            LsaDeregisterLogonProcess(data->LogonHandle);
+            if (data->cc_name)
+                free(data->cc_name);
             free(data);
         }
         free(id);
@@ -2649,6 +2643,15 @@ krb5_lcc_store(krb5_context context, krb5_ccache id, krb5_creds *creds)
     if (!is_windows_2000())
         return KRB5_FCC_NOFILE;
 
+    if (krb5_is_config_principal(context, creds->server)) {
+        /* mslsa cannot store config creds, so we have to bail.
+         * The 'right' thing to do would be to return an appropriate error,
+         * but that would require modifying the calling code to check
+         * for that error and ignore it.
+         */
+        return KRB5_OK;
+    }
+
 #ifdef KERB_SUBMIT_TICKET
     /* we can use the new KerbSubmitTicketMessage to store the ticket */
     if (KerbSubmitTicket( data->LogonHandle, data->PackageId, context, creds ))
@@ -2735,6 +2738,58 @@ krb5_lcc_get_flags(krb5_context context, krb5_ccache id, krb5_flags *flags)
     return KRB5_OK;
 }
 
+struct krb5int_lcc_iterator {
+    int id;
+};
+
+static krb5_error_code KRB5_CALLCONV
+krb5_lcc_ptcursor_new(krb5_context context, krb5_cc_ptcursor *cursor)
+{
+    krb5_cc_ptcursor new_cursor = (krb5_cc_ptcursor )malloc(sizeof(*new_cursor));
+    if (!new_cursor)
+        return ENOMEM;
+    new_cursor->ops = &krb5_lcc_ops;
+    new_cursor->data = (krb5_pointer)(1);
+    *cursor = new_cursor;
+    new_cursor = NULL;
+    return 0;
+}
+
+static krb5_error_code KRB5_CALLCONV
+krb5_lcc_ptcursor_next(krb5_context context, krb5_cc_ptcursor cursor, krb5_ccache *ccache)
+{
+    krb5_error_code code = 0;
+    *ccache = 0;
+    if (cursor->data == NULL)
+        return 0;
+
+    cursor->data = NULL;
+    if ((code = krb5_lcc_resolve(context, ccache, ""))) {
+        if (code != KRB5_FCC_NOFILE)
+            /* Note that we only want to return serious errors.
+             * Any non-zero return code will prevent the cccol iterator
+             * from advancing to the next ccache collection. */
+            return code;
+    }
+    return 0;
+}
+
+static krb5_error_code KRB5_CALLCONV
+krb5_lcc_ptcursor_free(krb5_context context, krb5_cc_ptcursor *cursor)
+{
+    if (*cursor) {
+        free(*cursor);
+        *cursor = NULL;
+    }
+    return 0;
+}
+
+static krb5_error_code KRB5_CALLCONV
+krb5_lcc_switch_to(krb5_context context, krb5_ccache id)
+{
+    return 0;
+}
+
 const krb5_cc_ops krb5_lcc_ops = {
     0,
     "MSLSA",
@@ -2753,11 +2808,14 @@ const krb5_cc_ops krb5_lcc_ops = {
     krb5_lcc_remove_cred,
     krb5_lcc_set_flags,
     krb5_lcc_get_flags,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
+    krb5_lcc_ptcursor_new,
+    krb5_lcc_ptcursor_next,
+    krb5_lcc_ptcursor_free,
+    NULL, /* move */
+    NULL, /* lastchange */
+    NULL, /* wasdefault */
+    NULL, /* lock */
+    NULL, /* unlock */
+    krb5_lcc_switch_to,
 };
 #endif /* _WIN32 */
