@@ -51,7 +51,7 @@
  */
 #define KRB5_MAJOR_RELEASE 1
 #define KRB5_MINOR_RELEASE 10
-#define KRB5_PATCHLEVEL 2
+#define KRB5_PATCHLEVEL 3
 /* #undef KRB5_RELTAIL */
-#define KRB5_RELDATE "20120531"
-#define KRB5_RELTAG "tags/krb5-1.10.2-final"
+#define KRB5_RELDATE "20120808"
+#define KRB5_RELTAG "krb5-1.10.3-final"
